import { __rest, __assign } from 'tslib';
import React, { forwardRef, useState, useEffect } from 'react';
import { ReactCanvasPatternLock } from 'react-canvas-pattern-lock';
import cn from 'classnames';
import { ButtonMobile } from '@alfalab/core-components-button/esm/mobile';
import { getDataTestId } from '@alfalab/core-components-shared/esm';
import { THEME_STATE, DEFAULT_EXTRA_BOUNDS, OBSERVE_OPTIONS, OBSERVABLE_TOKENS } from '../../consts.js';
import { getSizes, getTheme, getColorByToken, getDefaultObserveTarget } from '../../utils.js';

var commonStyles = {"component":"pattern-lock-v1__component_1b4f1","hidden":"pattern-lock-v1__hidden_1b4f1","message":"pattern-lock-v1__message_1b4f1","error":"pattern-lock-v1__error_1b4f1","forgotBtn":"pattern-lock-v1__forgotBtn_1b4f1"};
require('./index.css')

var BasePatternLock = forwardRef(function (_a, ref) {
    var _b, _c;
    var _d = _a.observeTokens, observeTokens = _d === void 0 ? false : _d, _e = _a.observerParams, observerParams = _e === void 0 ? {} : _e, _f = _a.justifyNodes, justifyNodes = _f === void 0 ? 'space-between' : _f, className = _a.className, error = _a.error, dataTestId = _a.dataTestId, _g = _a.forgotCodeBtnText, forgotCodeBtnText = _g === void 0 ? 'Забыли код?' : _g, _h = _a.showForgotCodeBtn, showForgotCodeBtn = _h === void 0 ? false : _h, onForgotBtnClick = _a.onForgotBtnClick, _j = _a.extraBounds, extraBounds = _j === void 0 ? DEFAULT_EXTRA_BOUNDS : _j, message = _a.message, messageClassName = _a.messageClassName, hover = _a.hover, _k = _a.styles, styles = _k === void 0 ? {} : _k, restProps = __rest(_a, ["observeTokens", "observerParams", "justifyNodes", "className", "error", "dataTestId", "forgotCodeBtnText", "showForgotCodeBtn", "onForgotBtnClick", "extraBounds", "message", "messageClassName", "hover", "styles"]);
    var _l = useState(), params = _l[0], setParams = _l[1];
    useEffect(function () {
        var _a = getSizes(), elementSizes = _a.elementSizes, width = _a.width, height = _a.height;
        setParams({
            theme: getTheme(elementSizes),
            width: width,
            height: height,
        });
    }, []);
    useEffect(function () {
        var styleObserver = null;
        if (observeTokens) {
            var _a = observerParams.options, options = _a === void 0 ? OBSERVE_OPTIONS : _a, _b = observerParams.getTarget, getTarget = _b === void 0 ? getDefaultObserveTarget : _b;
            styleObserver = new MutationObserver(function () {
                return setParams(function (prevState) {
                    var _a = getSizes(), width = _a.width, height = _a.height, elementSizes = _a.elementSizes;
                    if (!prevState) {
                        return { theme: getTheme(elementSizes), width: width, height: height };
                    }
                    var prevBgColor = prevState.theme[THEME_STATE.INITIAL].colors.bg;
                    var themeChanged = prevBgColor !== getColorByToken(OBSERVABLE_TOKENS.BG);
                    if (themeChanged)
                        return __assign(__assign({}, prevState), { theme: getTheme(elementSizes) });
                    return prevState;
                });
            });
            styleObserver.observe(getTarget(), options);
        }
        return function () { return styleObserver === null || styleObserver === void 0 ? void 0 : styleObserver.disconnect(); };
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [observeTokens]);
    var renderMessage = function () { return (React.createElement("div", { className: cn(commonStyles.message, messageClassName), "data-test-id": getDataTestId(dataTestId, 'message') }, message)); };
    var renderError = function () { return (React.createElement("div", { className: cn(commonStyles.message, commonStyles.error, messageClassName), "data-test-id": getDataTestId(dataTestId, 'error') }, error)); };
    return (React.createElement("div", { className: cn(commonStyles.component, styles.component, className, (_b = {},
            _b[commonStyles.hidden] = !params,
            _b)), "data-test-id": dataTestId },
        error ? renderError() : renderMessage(),
        React.createElement(ReactCanvasPatternLock, __assign({}, restProps, params, { ref: ref, rows: 3, cols: 3, justifyNodes: justifyNodes, extraBounds: extraBounds, hover: hover })),
        showForgotCodeBtn ? (React.createElement(ButtonMobile, { view: 'transparent', className: cn(commonStyles.forgotBtn, styles.forgotBtn), onClick: onForgotBtnClick, dataTestId: getDataTestId(dataTestId, 'forgot-code-btn') }, forgotCodeBtnText)) : (React.createElement("div", { className: cn(commonStyles.forgotBtn, styles.forgotBtn, (_c = {},
                _c[styles.hiddenBtn] = Boolean(styles.hiddenBtn),
                _c)) }))));
});

export { BasePatternLock };
